package com.ruoyi.ss.vipLevel.service.impl;

import com.ruoyi.common.utils.collection.CollectionUtils;
import com.ruoyi.ss.store.domain.StoreNameVO;
import com.ruoyi.ss.store.domain.StoreQuery;
import com.ruoyi.ss.store.service.StoreService;
import com.ruoyi.ss.vipLevel.domain.VipLevelVO;
import com.ruoyi.ss.vipLevel.service.VipLevelAssembler;
import com.ruoyi.ss.vipLevelSku.domain.VipLevelSkuQuery;
import com.ruoyi.ss.vipLevelSku.domain.VipLevelSkuVO;
import com.ruoyi.ss.vipLevelSku.domain.enums.VipLevelSkuStatus;
import com.ruoyi.ss.vipLevelSku.service.VipLevelSkuService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

@Service
public class VipLevelAssemblerImpl implements VipLevelAssembler {

    @Autowired
    private StoreService storeService;

    @Autowired
    private VipLevelSkuService vipLevelSkuService;

    @Override
    public void assembleStoreList(List<VipLevelVO> list) {
        if (CollectionUtils.isEmptyElement(list)) {
            return;
        }

        List<Long> storeIds = list.stream().map(VipLevelVO::getStoreIds).flatMap(Collection::stream).collect(Collectors.toList());
        List<StoreNameVO> storeList = new ArrayList<>();
        if (CollectionUtils.isNotEmptyElement(storeIds)) {
            StoreQuery query = new StoreQuery();
            query.setStoreIds(storeIds);
            storeList = storeService.selectNameList(query);
        }

        for (VipLevelVO vipLevel : list) {
            List<StoreNameVO> vipLevelStoreList = storeList.stream()
                .filter(store -> vipLevel.getStoreIds().contains(store.getStoreId()))
                .collect(Collectors.toList());
            vipLevel.setStoreList(vipLevelStoreList);
        }
    }

    @Override
    public void assembleSkuList(List<VipLevelVO> list) {
        if (CollectionUtils.isEmptyElement(list)) {
            return;
        }

        List<Long> levelIds = list.stream().map(VipLevelVO::getId).collect(Collectors.toList());

        Map<Long, List<VipLevelSkuVO>> group = vipLevelSkuService.selectByLevelIds(levelIds)
            .stream().collect(Collectors.groupingBy(VipLevelSkuVO::getLevelId));

        for (VipLevelVO vipLevel : list) {
            List<VipLevelSkuVO> vipLevelSkuList = group.get(vipLevel.getId());
            if (vipLevelSkuList == null) {
                vipLevelSkuList = Collections.emptyList();
            }
            vipLevel.setSkuList(vipLevelSkuList);
        }
    }

    @Override
    public void assembleSaleSkuList(List<VipLevelVO> list) {
        if (CollectionUtils.isEmptyElement(list)) {
            return;
        }

        List<Long> levelIds = list.stream().map(VipLevelVO::getId).collect(Collectors.toList());

        VipLevelSkuQuery query = new VipLevelSkuQuery();
        query.setLevelIds(levelIds);
        query.setStatus(VipLevelSkuStatus.ON_SALE.getStatus());
        Map<Long, List<VipLevelSkuVO>> group = vipLevelSkuService.selectVipLevelSkuList(query)
                .stream().collect(Collectors.groupingBy(VipLevelSkuVO::getLevelId));

        for (VipLevelVO vipLevel : list) {
            List<VipLevelSkuVO> vipLevelSkuList = group.get(vipLevel.getId());
            if (vipLevelSkuList == null) {
                vipLevelSkuList = Collections.emptyList();
            }
            vipLevel.setSkuList(vipLevelSkuList);
        }

    }
}
