package com.ruoyi.ss.vipLevelSku.service.impl;

import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.common.utils.collection.CollectionUtils;
import com.ruoyi.ss.vipLevel.domain.VipLevelVO;
import com.ruoyi.ss.vipLevelSku.domain.VipLevelSku;
import com.ruoyi.ss.vipLevelSku.domain.VipLevelSkuQuery;
import com.ruoyi.ss.vipLevelSku.domain.VipLevelSkuVO;
import com.ruoyi.ss.vipLevelSku.mapper.VipLevelSkuMapper;
import com.ruoyi.ss.vipLevelSku.service.VipLevelSkuService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Collections;
import java.util.Date;
import java.util.List;

/**
 * VIP等级定价Service业务层处理
 *
 * @author ruoyi
 * @date 2025-01-16
 */
@Service
public class VipLevelSkuServiceImpl implements VipLevelSkuService
{
    @Autowired
    private VipLevelSkuMapper vipLevelSkuMapper;

    /**
     * 查询VIP等级定价
     *
     * @param id VIP等级定价主键
     * @return VIP等级定价
     */
    @Override
    public VipLevelSkuVO selectVipLevelSkuById(Long id)
    {
        return vipLevelSkuMapper.selectVipLevelSkuById(id);
    }

    /**
     * 查询VIP等级定价列表
     *
     * @param vipLevelSku VIP等级定价
     * @return VIP等级定价
     */
    @Override
    public List<VipLevelSkuVO> selectVipLevelSkuList(VipLevelSkuQuery vipLevelSku)
    {
        return vipLevelSkuMapper.selectVipLevelSkuList(vipLevelSku);
    }

    /**
     * 新增VIP等级定价
     *
     * @param vipLevelSku VIP等级定价
     * @return 结果
     */
    @Override
    public int insertVipLevelSku(VipLevelSku vipLevelSku)
    {
        vipLevelSku.setCreateTime(DateUtils.getNowDate());
        return vipLevelSkuMapper.insertVipLevelSku(vipLevelSku);
    }

    /**
     * 修改VIP等级定价
     *
     * @param vipLevelSku VIP等级定价
     * @return 结果
     */
    @Override
    public int updateVipLevelSku(VipLevelSku vipLevelSku)
    {
        vipLevelSku.setUpdateTime(DateUtils.getNowDate());
        return vipLevelSkuMapper.updateVipLevelSku(vipLevelSku);
    }

    /**
     * 批量删除VIP等级定价
     *
     * @param ids 需要删除的VIP等级定价主键
     * @return 结果
     */
    @Override
    public int deleteVipLevelSkuByIds(List<Long> ids)
    {
        return vipLevelSkuMapper.deleteVipLevelSkuByIds(ids);
    }

    /**
     * 删除VIP等级定价信息
     *
     * @param id VIP等级定价主键
     * @return 结果
     */
    @Override
    public int deleteVipLevelSkuById(Long id)
    {
        return vipLevelSkuMapper.deleteVipLevelSkuById(id);
    }

    @Override
    public List<VipLevelSkuVO> selectByLevelId(Long levelId) {
        if (levelId == null) {
            return Collections.emptyList();
        }
        VipLevelSkuQuery query = new VipLevelSkuQuery();
        query.setLevelId(levelId);
        return vipLevelSkuMapper.selectVipLevelSkuList(query);
    }

    @Override
    public int batchInsert(List<? extends VipLevelSku> list) {
        if (CollectionUtils.isEmptyElement(list)) {
            return 0;
        }
        Date now = DateUtils.getNowDate();
        for (VipLevelSku sku : list) {
            sku.setCreateTime(now);
        }
        return vipLevelSkuMapper.batchInsert(list);
    }

    @Override
    public int batchUpdate(List<? extends VipLevelSku> list) {
        if (CollectionUtils.isEmptyElement(list)) {
            return 0;
        }
        Date now = DateUtils.getNowDate();
        for (VipLevelSku sku : list) {
            sku.setUpdateTime(now);
        }
        return vipLevelSkuMapper.batchUpdate(list);
    }

    @Override
    public List<VipLevelSkuVO> selectByLevelIds(List<Long> levelIds) {
        if (CollectionUtils.isEmptyElement(levelIds)) {
            return Collections.emptyList();
        }
        VipLevelSkuQuery query = new VipLevelSkuQuery();
        query.setLevelIds(levelIds);
        return vipLevelSkuMapper.selectVipLevelSkuList(query);
    }

    @Override
    public void afterCheck(VipLevelVO vo) {
        ServiceUtil.assertion(vo == null, "校验失败，数据不存在");

        // TODO 校验SKU限制类型与次数是否一致
    }
}
