package com.ruoyi.ss.vipOrder.service.impl;

import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.ss.channel.service.ChannelService;
import com.ruoyi.ss.user.domain.SmUserVO;
import com.ruoyi.ss.user.service.UserService;
import com.ruoyi.ss.vipLevel.domain.VipLevelVO;
import com.ruoyi.ss.vipLevel.service.VipLevelService;
import com.ruoyi.ss.vipLevelSku.domain.VipLevelSkuVO;
import com.ruoyi.ss.vipLevelSku.service.VipLevelSkuService;
import com.ruoyi.ss.vipOrder.domain.VipOrder;
import com.ruoyi.ss.vipOrder.domain.VipOrderQuery;
import com.ruoyi.ss.vipOrder.domain.bo.AddPayVipOrderBO;
import com.ruoyi.ss.vipOrder.domain.dto.AddPayVipOrderDTO;
import com.ruoyi.ss.vipOrder.domain.enums.VipOrderStatus;
import com.ruoyi.ss.vipOrder.service.VipOrderConverter;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.time.LocalDateTime;

/**
 * @author wjh
 * 2025/1/17
 */
@Service
public class VipOrderConverterImpl implements VipOrderConverter {

    @Autowired
    private ChannelService channelService;

    @Autowired
    private VipLevelSkuService vipLevelSkuService;

    @Autowired
    private VipLevelService vipLevelService;

    @Autowired
    private UserService userService;

    @Override
    public AddPayVipOrderBO toBO(AddPayVipOrderDTO dto) {
        if (dto == null) {
            return null;
        }

        AddPayVipOrderBO bo = new AddPayVipOrderBO();
        bo.setDto(dto);
        // 支付渠道
        bo.setChannel(channelService.selectSmChannelByChannelId(dto.getChannelId()));

        // sku
        VipLevelSkuVO sku = vipLevelSkuService.selectVipLevelSkuById(dto.getSkuId());
        bo.setSku(sku);
        if (sku != null && sku.getLevelId() != null) {
            // 会员等级
            VipLevelVO level = vipLevelService.selectVipLevelById(sku.getLevelId());
            bo.setLevel(level);

            if (level != null && level.getMchId() != null) {
                // 商户
                bo.setMch(userService.selectSmUserByUserId(level.getMchId()));
            }
        }

        // 下单用户
        bo.setUser(userService.selectSmUserByUserId(SecurityUtils.getUserId()));

        return bo;
    }

    @Override
    public VipOrder toOrder(AddPayVipOrderBO bo) {
        if (bo == null) {
            return null;
        }

        VipOrder order = new VipOrderQuery();
        // 用户数据
        SmUserVO user = bo.getUser();
        if (user != null) {
            order.setUserId(user.getUserId());
        }
        // 商户数据
        SmUserVO mch = bo.getMch();
        if (mch != null) {
            order.setMchId(mch.getUserId());
        }
        // 等级数据
        VipLevelVO level = bo.getLevel();
        if (level != null) {
            order.setLevelId(level.getId());
            order.setLevelName(level.getName());
            order.setLevelDiscount(level.getDiscount());
            order.setLevelStoreIds(level.getStoreIds());
        }
        // 解决方案
        AddPayVipOrderDTO dto = bo.getDto();
        if (dto != null) {
            order.setLevelSolution(dto.getSolution());
        }
        // sku数据
        VipLevelSkuVO sku = bo.getSku();
        if (sku != null) {
            order.setSkuId(sku.getId());
            order.setSkuTime(sku.getTime());
            order.setSkuPrice(sku.getPrice());
            order.setSkuLimitType(sku.getLimitType());
            order.setSkuLimitCount(sku.getLimitCount());
        }
        // 基础数据
        BigDecimal amount = sku == null || sku.getPrice() == null ? BigDecimal.ZERO : sku.getPrice();
        order.setAmount(amount);
        order.setExpireTime(LocalDateTime.now().plusMinutes(15));   // 15分钟内过期
        order.setStatus(VipOrderStatus.WAIT_PAY.getStatus());

        return order;
    }
}
