package com.ruoyi.ss.vipOrder.service.impl;

import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.common.utils.SnowFlakeUtil;
import com.ruoyi.ss.payBill.domain.PayBill;
import com.ruoyi.ss.payBill.domain.vo.DoPayVO;
import com.ruoyi.ss.payBill.service.PayBillConverter;
import com.ruoyi.ss.payBill.service.PayBillService;
import com.ruoyi.ss.vipOrder.domain.VipOrder;
import com.ruoyi.ss.vipOrder.domain.VipOrderQuery;
import com.ruoyi.ss.vipOrder.domain.VipOrderVO;
import com.ruoyi.ss.vipOrder.domain.bo.AddPayVipOrderBO;
import com.ruoyi.ss.vipOrder.domain.vo.AddAndPayVO;
import com.ruoyi.ss.vipOrder.mapper.VipOrderMapper;
import com.ruoyi.ss.vipOrder.service.VipOrderConverter;
import com.ruoyi.ss.vipOrder.service.VipOrderService;
import com.ruoyi.ss.vipOrder.service.VipOrderValidator;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.support.TransactionTemplate;

import java.util.List;

/**
 * 会员订单Service业务层处理
 *
 * @author ruoyi
 * @date 2025-01-16
 */
@Service
public class VipOrderServiceImpl implements VipOrderService
{
    @Autowired
    private VipOrderMapper vipOrderMapper;

    @Autowired
    private VipOrderValidator vipOrderValidator;

    @Autowired
    private VipOrderConverter vipOrderConverter;

    @Autowired
    private PayBillConverter payBillConverter;

    @Autowired
    private PayBillService payBillService;

    @Autowired
    private TransactionTemplate transactionTemplate;

    /**
     * 查询会员订单
     *
     * @param id 会员订单主键
     * @return 会员订单
     */
    @Override
    public VipOrderVO selectVipOrderById(Long id)
    {
        return vipOrderMapper.selectVipOrderById(id);
    }

    /**
     * 查询会员订单列表
     *
     * @param vipOrder 会员订单
     * @return 会员订单
     */
    @Override
    public List<VipOrderVO> selectVipOrderList(VipOrderQuery vipOrder)
    {
        return vipOrderMapper.selectVipOrderList(vipOrder);
    }

    /**
     * 新增会员订单
     *
     * @param vipOrder 会员订单
     * @return 结果
     */
    @Override
    public int insertVipOrder(VipOrder vipOrder)
    {
        vipOrder.setCreateTime(DateUtils.getNowDate());
        vipOrder.setOrderNo(String.valueOf(SnowFlakeUtil.newId()));
        return vipOrderMapper.insertVipOrder(vipOrder);
    }

    /**
     * 修改会员订单
     *
     * @param vipOrder 会员订单
     * @return 结果
     */
    @Override
    public int updateVipOrder(VipOrder vipOrder)
    {
        return vipOrderMapper.updateVipOrder(vipOrder);
    }

    /**
     * 批量删除会员订单
     *
     * @param ids 需要删除的会员订单主键
     * @return 结果
     */
    @Override
    public int deleteVipOrderByIds(Long[] ids)
    {
        return vipOrderMapper.deleteVipOrderByIds(ids);
    }

    /**
     * 删除会员订单信息
     *
     * @param id 会员订单主键
     * @return 结果
     */
    @Override
    public int deleteVipOrderById(Long id)
    {
        return vipOrderMapper.deleteVipOrderById(id);
    }

    @Override
    public AddAndPayVO addAndPay(AddPayVipOrderBO bo) {
        // 校验
        vipOrderValidator.preAddAndPay(bo);

        // 结果
        AddAndPayVO result = new AddAndPayVO();

        // 转为订单数据
        VipOrder order = vipOrderConverter.toOrder(bo);
        ServiceUtil.assertion(order == null, "订单数据为空");
        // 订单数据加入结果集和上下文
        result.setOrder(order);
        bo.setOrder(order);

        // 操作数据库
        transactionTemplate.execute(status -> {
            // 创建订单
            int insert = this.insertVipOrder(order);
            ServiceUtil.assertion(insert != 1, "创建订单失败");

            // 创建并支付
            PayBill pay = payBillConverter.toPoByVipOrderBO(bo);
            ServiceUtil.assertion(pay == null, "支付单数据为空");
            DoPayVO payResult = payBillService.createPayBill(pay);
            ServiceUtil.assertion(payResult == null, "调起支付失败");
            result.setPayParams(payResult.getPayParams());

            return insert;
        });

        return result;
    }
}
