package com.ruoyi.ss.vipOrder.service.impl;

import com.ruoyi.common.utils.MathUtils;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.common.utils.collection.CollectionUtils;
import com.ruoyi.ss.channel.domain.ChannelVO;
import com.ruoyi.ss.user.domain.SmUserVO;
import com.ruoyi.ss.vip.domain.VipQuery;
import com.ruoyi.ss.vip.domain.VipVO;
import com.ruoyi.ss.vip.service.VipService;
import com.ruoyi.ss.vipLevel.domain.VipLevelVO;
import com.ruoyi.ss.vipLevelSku.domain.VipLevelSkuVO;
import com.ruoyi.ss.vipLevelSku.domain.enums.VipLevelSkuStatus;
import com.ruoyi.ss.vipOrder.domain.bo.AddPayVipOrderBO;
import com.ruoyi.ss.vipOrder.domain.dto.AddPayVipOrderDTO;
import com.ruoyi.ss.vipOrder.service.VipOrderValidator;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Objects;

/**
 * @author wjh
 * 2025/1/17
 */
@Service
public class VipOrderValidatorImpl implements VipOrderValidator {

    @Autowired
    private VipService vipService;

    @Override
    public void preAddAndPay(AddPayVipOrderBO bo) {
        ServiceUtil.assertion(bo == null, "参数错误");

        AddPayVipOrderDTO dto = bo.getDto();
        VipLevelVO level = bo.getLevel();
        VipLevelSkuVO sku = bo.getSku();
        SmUserVO mch = bo.getMch();
        ChannelVO channel = bo.getChannel();
        SmUserVO user = bo.getUser();

        // 校验用户前端查看到的数据和后台数据是否一致
        ServiceUtil.assertion(sku == null, "ID为%sVIP等级价格不存在", dto.getSkuId());
        ServiceUtil.assertion(level == null, "的VIP等级不存在");
        ServiceUtil.assertion(!VipLevelSkuStatus.ON_SALE.getStatus().equals(sku.getStatus()), "ID为%s的SKU目前不可购买", sku.getId());
        ServiceUtil.assertion(!Objects.equals(dto.getSkuLimitType(), sku.getLimitType()), "当前使用限制已发生变化，请重新下单");
        ServiceUtil.assertion(!Objects.equals(dto.getSkuLimitCount(), sku.getLimitCount()), "当前限制次数已发生变化，请重新下单");
        ServiceUtil.assertion(sku.getPrice() == null, "价格不允许为空，请重新下单");
        ServiceUtil.assertion(!MathUtils.equals(dto.getSkuPrice(), sku.getPrice()), "当前价格已发生变化，请重新下单");
        ServiceUtil.assertion(!Objects.equals(dto.getSkuTime(), sku.getTime()), "当前可购买的时长已发生变化，请重新下单");
        ServiceUtil.assertion(!MathUtils.equals(dto.getLevelDiscount(), level.getDiscount()), "当前折扣已发生变化，请重新下单");
        ServiceUtil.assertion(!CollectionUtils.equals(dto.getLevelStoreIds(), level.getStoreIds()), "当前可用店铺已发生变化，请重新下单");

        // 商户是否存在
        ServiceUtil.assertion(mch == null, "商户不存在");
        // 用户是否存在
        ServiceUtil.assertion(user == null, "下单用户不存在");

        // 校验渠道是否可用
        ServiceUtil.assertion(channel == null, "ID为%s的支付渠道不存在", dto.getChannelId());
        ServiceUtil.assertion(channel.getEnabled() == null || !channel.getEnabled(), "ID为%s的支付渠道已禁用", channel.getChannelId());

        // 校验用户VIP兼容性，若不兼容，则校验是否有选择方案
        if (this.isCompatibility(sku, level, user)) {
            ServiceUtil.assertion(StringUtils.isBlank(dto.getSolution()), "当前拥有的VIP与下单的VIP不兼容，请选择解决方案");
        }
    }

    /**
     * 判断用户目前拥有的VIP与当前购买SKU是否兼容
     * @param sku sku
     * @param level 等级
     * @param user 用户
     */
    private boolean isCompatibility(VipLevelSkuVO sku, VipLevelVO level, SmUserVO user) {
        if (sku == null || level == null || user == null) {
            return false;
        }

        // 查询用户所拥有的当前等级的VIP
        VipQuery query = new VipQuery();
        query.setUserId(user.getUserId());
        query.setLevelId(level.getId());
        List<VipVO> vipList = vipService.selectVipList(query);
        if (CollectionUtils.isEmpty(vipList)) {
            return true;
        }

        // 逐一校验是否一致
        for (VipVO vip : vipList) {
            if (vip == null) {
                continue;
            }
            // 折扣
            if (!MathUtils.equals(vip.getDiscount(), level.getDiscount())) {
                return false;
            }
            // 可用店铺
            if (!CollectionUtils.equals(vip.getStoreIds(), level.getStoreIds())) {
                return false;
            }
            // 限制类型
            if (!Objects.equals(sku.getLimitType(), vip.getLimitType())) {
                return false;
            }
            // 限制次数
            if (!Objects.equals(sku.getLimitCount(), vip.getLimitCount())) {
                return false;
            }
        }

        return true;
    }
}
