package com.ruoyi.web.controller.ss;

import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.ValidGroup;
import com.ruoyi.common.core.domain.entity.SmUser;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.ss.user.domain.SmUserQuery;
import com.ruoyi.ss.user.domain.SmUserVO;
import com.ruoyi.ss.user.service.UserAssembler;
import com.ruoyi.ss.user.service.UserConverter;
import com.ruoyi.ss.user.service.UserService;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

/**
 * 普通用户信息Controller
 *
 * @author qzz
 * @date 2024-01-24
 */
@RestController
@RequestMapping("/system/smUser")
public class SmUserController extends BaseController
{
    @Autowired
    private UserService userService;

    @Autowired
    private UserAssembler userAssembler;

    @Autowired
    private UserConverter userConverter;

    /**
     * 查询普通用户信息列表
     */
    @PreAuthorize("@ss.hasPermi('system:smUser:list')")
    @GetMapping("/list")
    public TableDataInfo list(SmUserQuery smUser)
    {
        startPage();
        startOrderBy();
        List<SmUserVO> list = userService.selectSmUserList(smUser);
        userAssembler.assembleStoreCount(list);
        userAssembler.assembleDeviceCount(list);
        userAssembler.assembleRealServiceRate(list);
        userService.desensitization(list);
        return getDataTable(list);
    }

    /**
     * ids查询普通用户信息列表
     */
    @PreAuthorize("@ss.hasPermi('system:smUser:list')")
    @GetMapping("/listByIds/{ids}")
    public AjaxResult listByIds(@PathVariable Long[] ids)
    {
        List<SmUserVO> list = userService.selectByIds(Arrays.asList(ids));
        return success(list);
    }

    /**
     * 导出普通用户信息列表
     */
    @PreAuthorize("@ss.hasPermi('system:smUser:export')")
    @Log(title = "普通用户信息", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, SmUserQuery smUser)
    {
        List<SmUserVO> list = userService.selectSmUserList(smUser);
        ExcelUtil<SmUserVO> util = new ExcelUtil<SmUserVO>(SmUserVO.class);
        util.exportExcel(response, list, "普通用户信息数据");
    }

    /**
     * 获取普通用户信息详细信息
     */
    @PreAuthorize("@ss.hasPermi('system:smUser:query')")
    @GetMapping(value = "/{userId}")
    public AjaxResult getInfo(@PathVariable("userId") Long userId)
    {
        SmUserVO user = userService.selectSmUserByUserId(userId);
        List<SmUserVO> list = Collections.singletonList(user);
        userAssembler.assembleChannelList(list);
        userAssembler.assembleRealServiceRate(list);
        userService.desensitization(list);
        return success(user);
    }

    /**
     * 新增普通用户信息
     */
    @PreAuthorize("@ss.hasPermi('system:smUser:add')")
    @Log(title = "普通用户信息", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody @Validated(ValidGroup.Create.class) SmUser smUser)
    {
        SmUser po = userConverter.toPo(smUser);
        return toAjax(userService.insertSmUser(po));
    }

    /**
     * 修改普通用户信息
     */
    @PreAuthorize("@ss.hasPermi('system:smUser:edit')")
    @Log(title = "普通用户信息", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody @Validated(ValidGroup.Update.class) SmUser smUser)
    {
        SmUser po = userConverter.toPo(smUser);
        return toAjax(userService.updateSmUser(po));
    }

    @ApiOperation("重置用户服务费")
    @PreAuthorize("@ss.hasPermi('system:smUser:edit')")
    @Log(title = "普通用户信息", businessType = BusinessType.UPDATE)
    @PutMapping("/{userId}/resetService")
    public AjaxResult resetService(@PathVariable Long userId)
    {
        return toAjax(userService.resetService(userId));
    }

    /**
     * 删除普通用户信息
     */
    @PreAuthorize("@ss.hasPermi('system:smUser:remove')")
    @Log(title = "普通用户信息", businessType = BusinessType.DELETE)
	@DeleteMapping("/{userIds}")
    public AjaxResult remove(@PathVariable List<Long> userIds) {
        return toAjax(userService.logicDel(userIds));
    }

    /**
     * 删除普通用户信息
     */
    @PreAuthorize("@ss.hasPermi('system:smUser:unbindRealName')")
    @Log(title = "解除实名认证", businessType = BusinessType.OTHER)
	@PutMapping("/{userId}/resetRealName")
    public AjaxResult resetRealName(@PathVariable Long userId) {
        return toAjax(userService.resetRealName(userId));
    }

    // 获取用户统计信息
    @GetMapping("/userStatistics")
    public AjaxResult userStatistics(Long userId) {
        return success(userService.selectStatistics(userId));
    }

}
