package com.ruoyi.web.controller.ss;

import java.util.List;
import javax.servlet.http.HttpServletResponse;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.ss.vipOrder.domain.VipOrder;
import com.ruoyi.ss.vipOrder.domain.VipOrderVO;
import com.ruoyi.ss.vipOrder.domain.VipOrderQuery;
import com.ruoyi.ss.vipOrder.service.VipOrderService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 会员订单Controller
 *
 * @author ruoyi
 * @date 2025-01-16
 */
@RestController
@RequestMapping("/ss/vipOrder")
public class VipOrderController extends BaseController
{
    @Autowired
    private VipOrderService vipOrderService;

    /**
     * 查询会员订单列表
     */
    @PreAuthorize("@ss.hasPermi('ss:vipOrder:list')")
    @GetMapping("/list")
    public TableDataInfo list(VipOrderQuery query)
    {
        startPage();
        startOrderBy();
        List<VipOrderVO> list = vipOrderService.selectVipOrderList(query);
        return getDataTable(list);
    }

    /**
     * 导出会员订单列表
     */
    @PreAuthorize("@ss.hasPermi('ss:vipOrder:export')")
    @Log(title = "会员订单", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, VipOrderQuery query)
    {
        List<VipOrderVO> list = vipOrderService.selectVipOrderList(query);
        ExcelUtil<VipOrderVO> util = new ExcelUtil<VipOrderVO>(VipOrderVO.class);
        util.exportExcel(response, list, "会员订单数据");
    }

    /**
     * 获取会员订单详细信息
     */
    @PreAuthorize("@ss.hasPermi('ss:vipOrder:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(vipOrderService.selectVipOrderById(id));
    }

    /**
     * 新增会员订单
     */
    @PreAuthorize("@ss.hasPermi('ss:vipOrder:add')")
    @Log(title = "会员订单", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody VipOrder vipOrder)
    {
        return toAjax(vipOrderService.insertVipOrder(vipOrder));
    }

    /**
     * 修改会员订单
     */
    @PreAuthorize("@ss.hasPermi('ss:vipOrder:edit')")
    @Log(title = "会员订单", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody VipOrder vipOrder)
    {
        return toAjax(vipOrderService.updateVipOrder(vipOrder));
    }

    /**
     * 删除会员订单
     */
    @PreAuthorize("@ss.hasPermi('ss:vipOrder:remove')")
    @Log(title = "会员订单", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(vipOrderService.deleteVipOrderByIds(ids));
    }
}
